From de13595b43814d8b5106555545d7f0b107f3867b Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Mon, 15 Feb 2010 18:38:07 +0100
Subject: [PATCH] qtcapture: Support multiple devices.

---
 modules/access/qtcapture.m |   43 ++++++++++++++++++++++++++++++++++---------
 1 files changed, 34 insertions(+), 9 deletions(-)

diff --git a/modules/access/qtcapture.m b/modules/access/qtcapture.m
index f6665eb..0961181 100644
--- a/modules/access/qtcapture.m
+++ b/modules/access/qtcapture.m
@@ -111,7 +111,7 @@ vlc_module_end ()
         imageBufferToRelease = currentImageBuffer;
         currentImageBuffer = videoFrame;
         currentPts = (mtime_t)(1000000L / [sampleBuffer presentationTime].timeScale * [sampleBuffer presentationTime].timeValue);
-        
+
         /* Try to use hosttime of the sample if available, because iSight Pts seems broken */
         NSNumber *hosttime = (NSNumber *)[sampleBuffer attributeForKey:QTSampleBufferHostTimeAttribute];
         if( hosttime ) currentPts = (mtime_t)AudioConvertHostTimeToNanos([hosttime unsignedLongLongValue])/1000;
@@ -200,7 +200,7 @@ static int Open( vlc_object_t *p_this )
     /* Only when selected */
     if( *p_demux->psz_access == '\0' )
         return VLC_EGENERIC;
-    
+
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
     /* Set up p_demux */
@@ -209,24 +209,49 @@ static int Open( vlc_object_t *p_this )
     p_demux->info.i_update = 0;
     p_demux->info.i_title = 0;
     p_demux->info.i_seekpoint = 0;
-    
+
     p_demux->p_sys = p_sys = calloc( 1, sizeof( demux_sys_t ) );
     if( !p_sys )
         return VLC_ENOMEM;
-    
-    memset( &fmt, 0, sizeof( es_format_t ) );    
-    
+
+    memset( &fmt, 0, sizeof( es_format_t ) );
+
     QTCaptureDeviceInput * input = nil;
     NSError *o_returnedError;
 
-    p_sys->device = [QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeVideo];
+    if (EMPTY_STR(p_demux->psz_path))
+        p_sys->device = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo];
+    else
+    {
+        NSArray *devices = [QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeVideo];
+        NSString *deviceName = [NSString stringWithUTF8String:p_demux->psz_path];
+        for(QTCaptureDevice *device in devices)
+        {
+            if ([[device localizedDisplayName] localizedCaseInsensitiveCompare:deviceName] == NSOrderedSame)
+            {
+                p_sys->device = device;
+                break;
+            }
+        }
+        if (!p_sys->device)
+        {
+            msg_Err(p_demux, "Can't find your qtcapture device among:");
+
+            for(QTCaptureDevice *device in devices)
+            {
+                msg_Err(p_demux, "\t- '%s'", [[device localizedDisplayName] UTF8String]);
+            }
+            return VLC_ENOITEM;
+        }
+    }
+
     if( !p_sys->device )
     {
         dialog_FatalWait( p_demux, _("No Input device found"),
                         _("Your Mac does not seem to be equipped with a suitable input device. "
                           "Please check your connectors and drivers.") );
         msg_Err( p_demux, "Can't find any Video device" );
-        
+
         goto error;
     }
 
@@ -291,7 +316,7 @@ static int Open( vlc_object_t *p_this )
     NSLog( @"encoded_size %d %d", (int)encoded_size.width, (int)encoded_size.height );
     NSLog( @"display_size %d %d", (int)display_size.width, (int)display_size.height );
     NSLog( @"PAR size %d %d", (int)par_size.width, (int)par_size.height );
-    
+
     [p_sys->output setPixelBufferAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
         [NSNumber numberWithInt: p_sys->height], kCVPixelBufferHeightKey,
         [NSNumber numberWithInt: p_sys->width], kCVPixelBufferWidthKey,
-- 
1.6.4

